#include "global.h"


typedef struct IRentry {
    char	*fname;
    char	*period;
    char	*rname;
    time_t	mtime;
    int		uecount;
    double	bytes;
} IRentry;


int rcompIRentry(const struct IRentry *el1, const struct IRentry *el2);
int rcompIRentry(const struct IRentry *el1, const struct IRentry *el2) {
    int retval = 0, sorder = 1;

    if (islower((int)cfg.index_sort)) sorder = -1;
    switch (cfg.index_sort) {
bydate:
	sorder = 1;
	case 'd':
	case 'D':
		 retval = SAnumcmp(el1->mtime, el2->mtime);
		 break;
	case 'n':
	case 'N':
		 if (!(retval = strcmp(el1->rname, el2->rname))) goto bydate;
		 break;
	case 'b':
	case 'B':
		 if (!(retval = SAnumcmp(el1->bytes, el2->bytes))) goto bydate;
		 break;
	case 'u':
	case 'U':
		 if (!(retval = SAnumcmp(el1->uecount, el2->uecount))) goto bydate;
		 break;
    }

    return (sorder == 1) ? retval : -retval;
}


void ReportIndex() {
    DIR *TargetDir;
    FILE *IR, *tDF;
    struct dirent *tdirent;
    struct stat tstat;
    char *sReportDir = NULL, tfname[FILENAMESIZE], trname[FILENAMESIZE], tperiod[DATETIMESIZE * 2L + 1L];
    short int deCount, ircount = 0;
    time_t tsdate, tedate, curdate;
    int tuecount;
    double tbytes;
    struct IRentry *IRentries = NULL;


    /* IR: open target dir */
    if (!(TargetDir = opendir(cfg.target_directory))) SAexit("Can't open directory %s. %s", cfg.target_directory, strerror(errno));
    deCount = SAgetdirecount(TargetDir);

    /* Overwrite option: compute just generated reports directory */
    if (cfg.reports_overwrite) if ((sReportDir = strrchr(icfg.ReportDir, (int)'/') + 1L) == NULL) SAexit("Can't detect last directory for '%s'", icfg.ReportDir);
    /* Rotate option: compute current date and time */
    if (time(&curdate) == -1L) SAexit("Can't get current date. %s", strerror(errno));

    /* Process every directory entry as it is report directory */
    while ((tdirent = SAreaddir(TargetDir, deCount--)) != NULL) {
	/* Try to open description file using current directory entry as directory name */
	SAsnprintf(tfname, sizeof(tfname), "%s/%s/%s", cfg.target_directory, tdirent->d_name, icfg.DFname);
	/* If description file not present then skip */
	if (stat(tfname, &tstat) == -1) continue;
	
	/* tDF: open */
	tDF = SAfopenI("r", tfname);
	
	/* tDF: read */
	if (fscanf(tDF, "%ld %ld\n%511[^\n]\n%lf\n%d", &tsdate, &tedate, trname, &tbytes, &tuecount) != 5) {
	    SAwarning("Incorrect description file format, skipping '%s'", tfname);
	    SAfclose(tDF);
	    continue;
	}
	SAfclose(tDF);
	
	/* Compute period */
	SAstrlcpy(tperiod, GetDT(tsdate, "%x-"), sizeof(tperiod));
	SAstrlcat(tperiod, GetDT(tedate, "%x"), sizeof(tperiod));
	
	/* Overwrite option: if not just generated reports directory and period is same */
	/* Rotate option: if report is old enough */
	if (((cfg.reports_overwrite == 1) ? strcmp(sReportDir, tdirent->d_name) && !strcmp(icfg.Period, tperiod) : 0) ||
	    ((cfg.reports_overwrite == 2) ? strcmp(sReportDir, tdirent->d_name) && tsdate >= icfg.SDate && tedate <= icfg.EDate : 0) ||
	    (curdate - cfg.reports_rotate >= tstat.st_mtime)) {
	    SArm("%s/%s", cfg.target_directory, tdirent->d_name);
	    continue;
	}
	
	IRentries = (struct IRentry *)SArealloc((void *)IRentries, sizeof(struct IRentry) * (ircount + 1L));
	IRentries[ircount].fname = SAstrdup(tdirent->d_name);
	IRentries[ircount].period = SAstrdup(tperiod);
	IRentries[ircount].rname = SAstrdup(trname);
	IRentries[ircount].mtime = tstat.st_mtime;
	IRentries[ircount].uecount = tuecount;
	IRentries[ircount].bytes = tbytes;
	ircount++;
    }

    /* IR: sort */
    qsort(IRentries, (size_t)ircount, sizeof(struct IRentry), (int (*)(const void *, const void *))rcompIRentry);
    /* IR: open */
    IR = SAfopenH("./", 'r', loc->PSP, rcfg.IRsort, 0, TEMPL_REPORT, cfg.target_directory, icfg.IRname);
    /* IR: header */
    SAfprintf(IR, "<tr><th>%s<th>%s<th>%s<th>%s<th>%s\n", loc->Period, loc->GenerationD, loc->Users, loc->Bytes, loc->Configuration);
    /* IR: write */
    for (deCount = 0; deCount < ircount; deCount++) SAfprintf(IR, "<tr><td><a href=\"%s/%s.html\">%s</a><td>%s<td>%"SATSEP"d<td>%s<td class=lt>%s\n", IRentries[deCount].fname, icfg.LRname, IRentries[deCount].period, GetDT(IRentries[deCount].mtime, "%x %X"), IRentries[deCount].uecount, icfg.SAB(IRentries[deCount].bytes), IRentries[deCount].rname);
    /* IR: close */
    SAfcloseH(IR);
}
